@@ -151,10 +151,12 @@ where
     // type Future = Pin<Box<(dyn Future<Output = Result<Self::Response, Self::Error>> + Send + 'static)>>;
     type Future = future::Ready<Result<Self::Response, Self::Error>>;
 
+    #[inline]
     fn poll_ready(&mut self, _cx: &mut std::task::Context<'_>) -> std::task::Poll<Result<(), Self::Error>> {
         Ok(()).into()
     }
 
+    #[inline]
     fn call(&mut self, target: &T) -> Self::Future {
         let remote_addr = target.remote_addr();
         future::ok(HyperHandler {
@@ -167,6 +169,7 @@ where
 }
 
 impl From<Router> for Service {
+    #[inline]
     fn from(router: Router) -> Self {
         Service::new(router)
     }
@@ -238,12 +241,12 @@ impl HyperHandler {
                 if !is_allowed {
                     res.set_status_code(StatusCode::UNSUPPORTED_MEDIA_TYPE);
                 }
-            } else if res.body.is_none() {
-                //avoid warning when errors (404 etc.)
+            } else if res.body.is_none() && !has_error {
+                // check for avoid warning when errors (404 etc.)
                 tracing::warn!(
                     uri = ?req.uri(),
                     method = req.method().as_str(),
-                    "Http response content type header is not set"
+                    "Http response content type header not set"
                 );
             }
             if res.body.is_none() && has_error {
@@ -260,10 +263,8 @@ impl HyperHandler {
             }
             #[cfg(debug_assertions)]
             if let hyper::Method::HEAD = *req.method() {
-                if let Some(body) = res.body() {
-                    if !body.is_empty() {
-                        tracing::warn!("request with head method should have empty body: https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/HEAD");
-                    }
+                if !res.body.is_none() {
+                    tracing::warn!("request with head method should not have body: https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/HEAD");
                 }
             }
             res
@@ -276,9 +277,11 @@ impl hyper::service::Service<hyper::Request<hyper::body::Body>> for HyperHandler
     type Error = hyper::Error;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>> + Send>>;
 
+    #[inline]
     fn poll_ready(&mut self, _cx: &mut std::task::Context<'_>) -> std::task::Poll<Result<(), Self::Error>> {
         std::task::Poll::Ready(Ok(()))
     }
+    #[inline]
     fn call(&mut self, req: hyper::Request<hyper::body::Body>) -> Self::Future {
         let response = self.handle(req.into());
         let fut = async move {
